@@ -1,10 +1,9 @@
 package Net::DNS::Packet;
 
 #
-# $Id: Packet.pm 1488 2016-06-07 09:04:50Z willem $
+# $Id: Packet.pm 1709 2018-09-07 08:03:09Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1488 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1709 $)[1];
 
 
 =head1 NAME
@@ -28,6 +27,7 @@ A Net::DNS::Packet object represents a DNS protocol packet.
 
 
 use strict;
+use warnings;
 use integer;
 use Carp;
 
@@ -130,7 +130,7 @@ sub decode {
 			answer	   => [],
 			authority  => [],
 			additional => [],
-			answersize => length $$data
+			replysize  => length $$data
 			}, $class;
 
 		# question/zone section
@@ -191,8 +191,7 @@ sub encode {
 
 	my $edns = $self->edns;					# EDNS support
 	my @addl = grep !$_->isa('Net::DNS::RR::OPT'), @{$self->{additional}};
-	unshift( @addl, $edns ) if $edns->_specified;
-	$self->{additional} = \@addl;
+	$self->{additional} = [$edns, @addl] if $edns->_specified;
 
 	return $self->truncate($size) if $size;
 
@@ -241,6 +240,7 @@ sub edns {
 	my $link = \$self->{xedns};
 	($$link) = grep $_->isa(qw(Net::DNS::RR::OPT)), @{$self->{additional}} unless $$link;
 	$$link = new Net::DNS::RR( type => 'OPT' ) unless $$link;
+	return $$link;
 }
 
 
@@ -275,7 +275,11 @@ sub reply {
 	$header->rd( $qheadr->rd );				# copy these flags into reply
 	$header->cd( $qheadr->cd );
 
-	$reply->edns->size($UDPmax) if $query->edns->_specified;
+	return $reply unless grep $_->isa('Net::DNS::RR::OPT'), @{$query->{additional}};
+
+	my $edns = $reply->edns();
+	CORE::push( @{$reply->{additional}}, $edns );
+	$edns->size($UDPmax);
 	return $reply;
 }
 
@@ -379,9 +383,9 @@ sub string {
 	my $header = $self->header;
 	my $update = $header->opcode eq 'UPDATE';
 
-	my $server = $self->{answerfrom};
-	my $length = $self->{answersize};
-	my $string = $server ? ";; Answer received from $server ($length bytes)\n" : "";
+	my $server = $self->{replyfrom};
+	my $length = $self->{replysize};
+	my $string = $server ? ";; Response received from $server ($length bytes)\n" : "";
 
 	$string .= ";; HEADER SECTION\n" . $header->string;
 
@@ -412,26 +416,28 @@ sub string {
 }
 
 
-=head2 answerfrom
+=head2 from
 
-    print "packet received from ", $packet->answerfrom, "\n";
+    print "packet received from ", $packet->from, "\n";
 
 Returns the IP address from which this packet was received.
 User-created packets will return undef for this method.
 
 =cut
 
-sub answerfrom {
+sub from {
 	my $self = shift;
 
-	$self->{answerfrom} = shift if scalar @_;
-	$self->{answerfrom};
+	$self->{replyfrom} = shift if scalar @_;
+	$self->{replyfrom};
 }
 
+sub answerfrom { &from; }					# uncoverable pod
 
-=head2 answersize
 
-    print "packet size: ", $packet->answersize, " bytes\n";
+=head2 size
+
+    print "packet size: ", $packet->size, " bytes\n";
 
 Returns the size of the packet in bytes as it was received from a
 nameserver.  User-created packets will return undef for this method
@@ -439,10 +445,12 @@ nameserver.  User-created packets will return undef for this method
 
 =cut
 
-sub answersize {
-	shift->{answersize};
+sub size {
+	shift->{replysize};
 }
 
+sub answersize { &size; }					# uncoverable pod
+
 
 =head2 push
 
@@ -723,7 +731,7 @@ The truncate method takes a maximum length as argument and then tries
 to truncate the packet and set the TC bit according to the rules of
 RFC2181 Section 9.
 
-The minimum maximum length that is honoured is 512 octets.
+The smallest length limit that is honoured is 512 octets.
 
 =cut
 
@@ -756,8 +764,7 @@ sub truncate {
 	$size = UDPSZ unless $size > UDPSZ;
 	$size -= $sigrr->_size if $sigrr;
 
-	my $data = pack 'x' x 12;				# header placeholder
-	my $hdsz = length $data;
+	my $data = pack 'x' x HEADER_LENGTH;			# header placeholder
 	$self->{count} = [];
 
 	my $tc;
@@ -806,7 +813,7 @@ sub truncate {
 
 	my @part = qw(question answer authority additional);
 	my @size = map scalar( @{$self->{$_}} ), @part;
-	pack 'n6 a*', $self->header->id, $self->{status}, @size, substr( $data, $hdsz );
+	pack 'n6 a*', $self->header->id, $self->{status}, @size, substr( $data, HEADER_LENGTH );
 }
 
 
